#==========================================================================
# Copyright 2012 Lucidel, Inc., 2013 Cloudoscope Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================

from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect
from django.contrib import auth
from django.core.context_processors import csrf
from django.contrib.auth.forms import UserCreationForm


def home(request):
    if request.user.is_anonymous():
        return HttpResponseRedirect('/accounts/login/')
    else:
        return HttpResponseRedirect('/q/home/')


def login(request):
    c= {}
    c.update(csrf(request))
    return render_to_response('login.html', c)

def auth_view(request):
    username = request.POST.get('username', '')
    password = request.POST.get('password', '')
    user = auth.authenticate(username=username, password=password)

    if user is not None:
        auth.login(request, user)
        return HttpResponseRedirect('/accounts/loggedin')
    else:
        return HttpResponseRedirect('/accounts/invalid')

def loggedin(request):
    # return render_to_response('loggedin.html', {'full_name': request.user.username, 'the_user': request.user})
    return HttpResponseRedirect('/q/home')

def invalid_login(request):
    return render_to_response('invalid_login.html', {'the_user': request.user})

def logout(request):
    auth.logout(request)
    return render_to_response('logout.html', { 'the_user': request.user})

def register_user(request):
    if request.method == 'POST':
        form = UserCreationForm(request.POST)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect('/accounts/register_success')
        else:
            args = {}
            args.update(csrf(request))
            args['form'] = form
            args['the_user'] = request.user
            return render_to_response('register.html', args)

    args = {}
    args.update(csrf(request))
    args['form'] = UserCreationForm()
    args['the_user'] = request.user

    return render_to_response('register.html', args)

def register_success(request):
    return render_to_response('register_success.html')


#==========================================================================
# Copyright 2012 Lucidel, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================

from django.db.models.query_utils import Q
from .query import MongoQuery

class MongoQuerySet(object):
    
    def __init__(self, model=None, query=None):
        self.model = model
        self.query = query or MongoQuery(self.model)
        self.flat = False
        
    def __len__(self):
        return self.count()
    
    def __iter__(self):
        #result cache should be used
        return self.iterator()

    def __getitem__(self, k):
        """
        Retrieves an item or slice from the set of results.
        """
        if not isinstance(k, (slice, int, long)):
            raise TypeError
        
        cursor = self.cursor()
        
        if isinstance(k, (int, long)):
            return self._handle_result_item(cursor.__getitem__(k))
        else:
            return map(self._handle_result_item, cursor.__getitem__(k))
        
    def _clone(self):
        query = self.query.clone()
        c = self.__class__(self.model, query)
        c.flat = self.flat
        return c
    
    def _model_attributes(self):
        if not hasattr(self, '_model_attributes_cache'):
            attributes = {}
            
            for f in self.model._meta.fields:
                attr, column = f.get_attname_column()
                attributes[column] = (attr, f)
                
            self._model_attributes_cache = attributes
        
        return self._model_attributes_cache
    
    def _create_object_from_response(self, item):
        obj = self.model()
        #{db_column: [model_attribute, field],...}
        attributes = self._model_attributes()
        
        for column_name, value in item.items():
            #FIXME: datetime should be converted to date for datetime field
            try:
                attr, field = attributes[column_name]
                setattr(obj, attr, field.to_python(value))
            except KeyError:
                pass
        obj._state.adding = False
        return obj
    
    def _create_values_from_response(self, item):
        meta = self.model._meta
        fields = self.query.get_fields()
        
        if self.flat:
            f = meta.get_field(fields[0])
            return f.to_python(item[f.get_attname_column()[1]])
        else:
            output = {}
            for field_name in fields:
                f = meta.get_field(field_name)
                output[field_name] = f.to_python(item[f.get_attname_column()[1]])
            return output   
         
    def _handle_result_item(self, item):
        if self.query.fields:
            return self._create_values_from_response(item)
        else:
            return self._create_object_from_response(item)
    
    def cursor(self):
        return self.query.get_result()
    
    def iterator(self):
        #result cache should be used
        cursor = self.cursor()
        if cursor:
            for item in cursor:
                yield self._handle_result_item(item)
    
    def select_related(self, *fields, **kwargs):
        #FIXME: Does not work for now
        return self._clone()
    
    def get(self, *args, **kwargs):
        clone = self.filter(*args, **kwargs)
        if self.query.can_filter():
            clone = clone.order_by()

        num = len(clone)
        if num == 1:
            return clone[0]
        if not num:
            raise self.model.DoesNotExist("%s matching query does not exist."
                    % self.model._meta.object_name)
        raise self.model.MultipleObjectsReturned("get() returned more than one %s -- it returned %s! Lookup parameters were %s"
                % (self.model._meta.object_name, num, kwargs))

    def values_list(self, *args, **kwargs):
        qs = self._clone()
        qs.flat = bool(kwargs.get('flat', False))
        qs.query.set_fields(args)
        return qs
    
    def order_by(self, *args):
        qs = self._clone()
        qs.query.clear_ordering()        
        qs.query.add_ordering(args)
        return qs
    
    def count(self):
        return self.query.get_count()
    
    def using(self, using):
        return self._clone()

    def _using_collection_prefix(self, using_prefix=None):
        """
        This method enables redirecting the entire model onto another collection.
        Note: this should be used VERY carefully as it will redirect all reading and writing.
        Use using_prefix=None to cancel the redirection.
        E.g. if using_prefix='acme_' and the db name is 'customers', a new db will be used, called 'acme_customers'.
        """
        self.query._setTargetCollectionPrefix(using_prefix)

    def exists(self):
        return self.query.has_results()
    
    def exclude(self, *args, **kwargs):
        return self._filter_or_exclude(True, *args, **kwargs)
    
    def filter(self, *args, **kwargs):
        return self._filter_or_exclude(False, *args, **kwargs)
    
    def all(self, **kwargs):
        return self._clone()

    def delete(self):
        assert self.query.can_filter(), \
                "Cannot use 'limit' or 'offset' with delete."
                
        del_query = self._clone()
        del_query.query.delete()

    delete.alters_data = True
    
    def _filter_or_exclude(self, negate, *args, **kwargs):
        if args or kwargs:
            assert self.query.can_filter(), \
                    "Cannot filter a query once a slice has been taken."

        clone = self._clone()
        if negate:
            clone.query.add_q(~Q(*args, **kwargs))
        else:
            clone.query.add_q(Q(*args, **kwargs))
        return clone
#        for filter, value in kwargs.items():
#            clone.query.add_q(MongoQ(negate, filter, value))
#        return clone
    
    def insert(self, items):
        if items:
            return self.query.insert(items)
        
    #These maybe should be delivered directly to query from __getattr__
    def ensure_index(self, *args, **kwargs):
        return self.query.collection.ensure_index(*args, **kwargs)
    
    def drop_indexes(self):
        return self.query.collection.drop_indexes()
    
    def index_information(self):
        return self.query.collection.index_information()
    
    def reindex(self):
        return self.query.collection.reindex()

    def complex_filter(self, filter_obj):
        """
        Returns a new QuerySet instance with filter_obj added to the filters.

        filter_obj can be a Q object (or anything with an add_to_query()
        method) or a dictionary of keyword lookup arguments.

        This exists to support framework features such as 'limit_choices_to',
        and usually it will be more natural to use other methods.
        """
        if isinstance(filter_obj, Q) or hasattr(filter_obj, 'add_to_query'):
            clone = self._clone()
            clone.query.add_q(filter_obj)
            return clone
        else:
            return self._filter_or_exclude(None, **filter_obj)

    def _update(self, values):
        assert self.query.can_filter(), \
                "Cannot update a query once a slice has been taken."        
        
        data = {}
        
        for field, model, value in values:
            data[field.name] = value

        self.query.update(data)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

#==========================================================================
# Copyright 2012 Lucidel, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================

__author__ = 'Gadi Oren'
__email__ = 'gadi.oren.1@gmail.com'
__version__ = '0.1.1'





#==========================================================================
# Copyright 2012 Lucidel, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================

from django.db import models
import bson
from djangotoolbox.fields import ListField, DictField

class MongoIdField(models.CharField, models.AutoField):
    
    def __init__(self, **kwargs):
        kwargs['max_length'] = 24
        kwargs['blank'] = True
        kwargs['primary_key'] = True
        kwargs['db_column'] = '_id'
        kwargs.setdefault('editable', False)
        super(MongoIdField, self).__init__(**kwargs)
        
    def get_internal_type(self):
        return models.CharField.__name__

    def contribute_to_class(self, cls, name):
        assert not cls._meta.has_auto_field, \
          "A model can't have more than one AutoField: %s %s %s; have %s" % \
           (self, cls, name, cls._meta.auto_field)
        super(MongoIdField, self).contribute_to_class(cls, name)
        cls._meta.has_auto_field = True
        cls._meta.auto_field = self
    
    def to_python(self, value):
        if isinstance(value, bson.objectid.ObjectId):
            return value
        
        return bson.objectid.ObjectId(str(value))
    
    def get_prep_lookup(self, lookup_type, value):
        if lookup_type in ('range', 'in'):
            return [self.get_prep_value(v) for v in value]
            
        return self.to_python(value)


class MongoDateTimeField(models.DateTimeField):
    def get_db_prep_value(self, value, connection, prepared=False):
        # Casts dates into the format expected by the backend
        if not prepared:
            value = self.get_prep_value(value)
        return value


class ValuesField(ListField):
    def formfield(self, **kwargs):
        return models.Field.formfield(self, StringListField, **kwargs)

    def _convert(self, func, values, *args, **kwargs):
        if isinstance(values, (list, tuple, set)):
            return self._type(func(value, *args, **kwargs) for value in values)
        if isinstance(values, (str,unicode)):
            split_values = values.replace('[','').replace(']','').replace(' ','').split(',')
            values_lst = [i for i in split_values if len(i)>0]
            return self._type(func(value, *args, **kwargs) for value in values_lst)
        return values


class StringListField(models.CharField):
    def prepare_value(self, value):
        return ', '.join(value)

    def to_python(self, value):
        if not value:
            return []
        return [item.strip() for item in value.split(',')]


class DictionaryField(DictField):
    def _convert(self, func, values, *args, **kwargs):
        if values is None:
            return None
        if isinstance(values, (str, unicode)):
            return eval(values)
        return dict((key, func(value, *args, **kwargs))
            for key, value in values.iteritems())


#==========================================================================
# Copyright 2012 Lucidel, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================

from types import DictType
from .exceptions import MongoQuerySetExeption
from datetime import datetime, date

def fix_value(value):

    if isinstance(value, datetime):
        return value

    if isinstance(value, date):
        return datetime(value.year, value.month, value.day)

    return value

class MongoQ(object):
    COLON = ":"
    NOT = "$ne"
    NE = "$ne"
    GT = "$gt"
    GTE = "$gte"
    LT = "$lt"
    LTE = "$lte"
    IN = "$in"
    NIN = "$nin"
    OR = "$or"
    NOR = "$nor"
    AND = "$and"
    REGEX = "$regex"
    IREGEX = "$regex"
    BSON_TYPE = "$type"
    EXISTS = "$exists"

    lookups = {
        'pk': None,
        'id': None,
        'not': NOT,
        'gt': GT,
        'gte': GTE,
        'lt': LT,
        'lte': LTE,
        'exact': None,
        'in': IN,
        'regex': REGEX,
        'iregex': IREGEX,
        'type' : BSON_TYPE,
        'exists' : EXISTS,
        }

    additional_options = {
        'iregex' : {'$options': 'i'}
    }

    negate_lookup = {
        COLON : NE,
        NE : COLON,
        GT : LTE,
        LTE : GT,
        GTE : LT,
        LT : GTE,
        IN : NIN,
        NIN : IN,
        OR : NOR,
        NOR : OR,
    }

    BSON_TYPES = {
        float : 1,
        str: 2,
        object: 3,
        list: 4,
        bool: 8,
        date: 9,
        datetime: 9,
        None: 10,
        int: 16,
        }

    def __init__(self, negate, filter, value):
        self.negate = negate
        self.filter = filter
        self._value = value
        self._dotFieldNotation = False

    def __deepcopy__(self, memodict):
        q = self.__class__(self.negate, self.filter, self._value)
        return q

    def _get_field_first_part(self):
        params = self.filter.split('__')
        name = params[0]
        return name

    def getDotNotationFieldName(self, model):
        lookup, params = self._analyze_field_lookups(model)
        field = self.getModelField(self.field, model)
        name = field.column
        if self._dotFieldNotation:
            name += '.'+params[1]
        return name

    @property
    def field(self):
        params = self.filter.split('__')
        name = params[0]
        return name

    def construct_spec(self, negate, filter, value, additional_options):
        if filter and filter != self.COLON:
            spec = {filter: value}
            if negate:
                spec = {self.negate_lookup[filter]: value}
            if additional_options:
                spec.update(additional_options)
        else:
            if negate:
                spec = {self.NE: value}
            else:
                spec = value
        return spec

    def _analyze_field_lookups(self, model):
        params = self.filter.split('__')
        if len(params) > 3:
            raise MongoQuerySetExeption('Does not support related objects filtering. Only be related field.')
        elif len(params) == 3:
            meta_field = model._meta.get_field(params[0])
            rel = meta_field.rel
            #this is field that saved as FK in MongoModel so we can filter by this field
            if rel:
                if params[1] == rel.field_name:
                    lookup = params[2]
                else:
                    raise MongoQuerySetExeption('Does not support related objects filtering. Only be related field.')
            elif meta_field._type is DictType:
                lookup = params[2]
                self._dotFieldNotation = True
            else:
                raise MongoQuerySetExeption('No related fields or dict.')
        elif len(params) == 2:
            lookup = params[1]
            if lookup in ('pk', 'id'): #hotfix. should check `rel` attribute of field
                lookup = 'exact'
        else:
            lookup = 'exact'
        return lookup, params

    def getModelField(self, field_name, model):
        if field_name == 'pk':
            field = model._meta.pk
        else:
            field = model._meta.get_field(field_name)
        return field

    def spec(self, model):
        lookup, params = self._analyze_field_lookups(model)

        if lookup not in self.lookups:
            raise MongoQuerySetExeption('Does not support "%s" lookup' % lookup)

        filter = None

        filter = self.lookups[lookup]

        field_name = self._get_field_first_part()
        field = self.getModelField(field_name, model)

        if lookup in self.lookups and self.lookups[lookup]  == self.EXISTS:
            value = self._value
        elif lookup in self.lookups and self.lookups[lookup]  == self.BSON_TYPE:
            value = self.BSON_TYPES[self._value]
        else:
            value = fix_value(field.get_prep_lookup(lookup, self._value))

        additional_options = {}
        if lookup in self.additional_options:
            additional_options = self.additional_options[lookup]

        spec = self.construct_spec(self.negate, filter, value, additional_options)

        return spec


def negate_where_clause(spec):
    res = spec
    if not isinstance(spec, dict):
        return res
    assert isinstance(spec, dict)

    operation = MongoQ.AND

    if len(spec) != 1:
        # original operation is AND. Using de-morgan, not(a AND b) ==> not(a) OR not(b)
        new_operation = MongoQ.OR
        or_items = []
        for k, v in spec.items():
            or_items.append(negate_where_clause({k:v}))
        res = {new_operation : or_items}
        return res

    assert len(spec.keys()) == 1

    field = spec.keys()[0]
    val = spec[field]

    if isinstance(val, dict):
        operation = val.keys()[0]
        val = val[operation]
    elif isinstance(val, list):
        operation = spec.keys()[0]
    else:
        operation = MongoQ.COLON

    new_operation = MongoQ.negate_lookup[operation]

    if new_operation == MongoQ.COLON:
        res = {field : val}
    elif new_operation in [MongoQ.OR, MongoQ.NOR]:
        res = {new_operation : val}
    else:
        res = {field : {new_operation:val}}

    return res



#==========================================================================
# Copyright 2012 Lucidel, Inc., 2013 Cloudoscope Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================

from django.contrib import admin
from models import UniqueVisit, TestModel1, TestModel2, TestModel3, AnalyticsAccount, Profile, TstAuthor, TstBook


class UniqueVisitAdmin(admin.ModelAdmin):
    list_display = ['account', 'date', 'visit_count', 'source', 'referral_path', 'keyword', 'campaign', 'landing_page_path', 'exit_page_path',
                    'user_id', 'time_on_site', 'page_views', 'first_visit_date', 'location', 'demographics', 'goal_values', 'goal_starts', 'goal_completions']
    list_filter = ['account', 'date', ]


class TestModel1Admin(admin.ModelAdmin):
    list_display = ['id', 'start_date', 'some_id', 'values',]

class TestModel2Admin(admin.ModelAdmin):
    list_display = ['id', 'start_date', 'some_id', 'values',]

class TestModel3Admin(admin.ModelAdmin):
    list_display = ['id', 'start_date', 'some_id', 'values',]

class AnalyticsAccountAdmin(admin.ModelAdmin):
    list_display = ['name', 'user', 'created_on', ]


class ProfileAdmin(admin.ModelAdmin):
    list_display = ['username', 'first_name', 'last_name', 'company_name' ]

class TstAuthorAdmin(admin.ModelAdmin):
    list_display = ['first_name', 'last_name',  ]

class TstBookAdmin(admin.ModelAdmin):
    list_display = ['name', 'author_info', 'publish_date',  ]

    def author_info(self, obj):
        mobj = TstAuthor.objects.get(id=obj.author_id)
        return mobj.last_name + ', ' + mobj.first_name

admin.site.register(UniqueVisit, UniqueVisitAdmin)
admin.site.register(TestModel1, TestModel1Admin)
admin.site.register(TestModel2, TestModel2Admin)
admin.site.register(TestModel3, TestModel3Admin)
admin.site.register(AnalyticsAccount, AnalyticsAccountAdmin)
admin.site.register(Profile, ProfileAdmin)
admin.site.register(TstAuthor, TstAuthorAdmin)
admin.site.register(TstBook, TstBookAdmin)



#==========================================================================
# Copyright 2012 Lucidel, Inc., 2013 Cloudoscope Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================
import base64
from django.contrib.auth.models import User
from django.conf import settings
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.template import RequestContext
from django.utils import timezone
from django.views.generic.simple import direct_to_template
from django.core.context_processors import csrf
from data.forms import QuetionEditForm, DocumentForm
from django.http import HttpResponseRedirect
from data.models import Question
from django.shortcuts import render_to_response
from django import forms
from django.contrib.auth.decorators import login_required

@login_required
def questions_home(request):
    user = request.user
    q_qs = Question.objects.all()
    if 'filter' in request.GET and request.GET['filter'] == 'user':
        q_qs = q_qs.filter(user=user)
    if 'sort' in request.GET and request.GET['sort'] == 'time':
        questions = [(q.date, q) for q in q_qs]
    else:
        questions = [((-len(q.vote_ids), q.date), q) for q in q_qs]
    questions = map(lambda x: x[1], sorted(questions))
    context = {'questions': questions, 'the_user': request.user}
    context.update(csrf(request))
    return render_to_response('question/home.html', context)


@login_required
def questions_search(request):
    user = request.user
    if request.method == 'POST':
        the_user=None
        if User.objects.filter(username=request.POST['search']).count() == 1:
            the_user = User.objects.get(username=request.POST['search'])
        questions = Question.objects.filter(Q(question__iregex=request.POST['search'])|Q(user=the_user)).order_by('date')
        context = {'questions': questions, 'the_user': request.user}
        context.update(csrf(request))
        return render_to_response('question/home.html', context)

@login_required
def questions_edit(request):
    context = {}
    form = QuetionEditForm(hide_condition=True)

    if request.method == 'POST':
        form= QuetionEditForm(request.POST, hide_condition=True)

        if form.is_valid():
            # form.save()
            txt = form.cleaned_data['question_txt']
            question_id = form.cleaned_data['question_id']

            if question_id == '':
                nq = Question(
                    user = request.user,
                    date = timezone.now(),
                    question = txt
                )
                nq.save()
                return HttpResponseRedirect('/q/media/?q=%s'% nq.id)
            else:
                question = Question.objects.get(id=question_id)
                question.question = txt
                question.save()
                questions = Question.objects.all()
                return HttpResponseRedirect('/q/media/?q=%s'% question.id)
        else:
            args = {}
            args.update(csrf(request))
            args['form'] = form
            return render_to_response('question/question_form.html', args)
    else:
        if 'q' in request.GET:
            question_id = request.GET['q']
            question = Question.objects.get(id=question_id)
            form = QuetionEditForm({'question_txt':question.question, 'question_id':question_id, }, hide_condition=True)
            context.update({'q': question_id})

    context.update({'form': form})
    context.update(csrf(request))


    return direct_to_template(request, 'question/question_form.html', context)



@login_required
def list_question_media(request):
    # Handle file upload
    context = {}
    documents = []

    if request.method == 'POST':
        form = DocumentForm(request.POST, request.FILES)
        if form.is_valid():
            docfile = request.FILES['docfile']
            question_id = form.cleaned_data['question_id']

            question = Question.objects.get(id=question_id)
            docfile_name = docfile.name
            docfile_name_changed = docfile_name.replace('.','~^~')
            question.docs.update({docfile_name_changed:docfile.content_type})
            question.image.update({docfile_name_changed +'_url':'/static/display/s_'+docfile_name, docfile_name_changed +'_name': docfile_name, docfile_name_changed +'_content_type':docfile.content_type})
            file_read = docfile.file.read()
            file_data = base64.b64encode(file_read)
            question.image.update({docfile_name_changed +'_data': file_data})
            # question.save()
            nq = Question(
                user = question.user,
                date = question.date,
                question = question.question,
                docs = question.docs,
                image = question.image,
                audio = question.audio,
                other = question.other,
                vote_ids = question.vote_ids,
            )
            question.delete() # todo - there is an update issue here
            nq.save()

            questions = Question.objects.all()
            return render_to_response('question/home.html', {'questions':questions, 'the_user': request.user, 'message': 'Media updated!' })

            # Redirect to the document list after POST
            # return HttpResponseRedirect(reverse('data.views.list_question_media'))
    else:
        if 'q' in request.GET:
            question_id = request.GET['q']
            form = DocumentForm({ 'question_id':question_id,}, hide_condition=True)
            question = Question.objects.get(id=question_id)
            documents = []
            for docname in question.docs:
                file_data = question.image[docname+'_data']
                file_name = question.image[docname+'_name']
                orig_fname = settings.STATIC_ROOT + '/../moma_example/static/display/' + file_name
                new_fname = settings.STATIC_ROOT + '/../moma_example/static/display/tn_' + file_name
                _decode_image_to_size(file_data, orig_fname, new_fname, basewidth=128)
                documents.append({'docfile':{'url':question.image[docname +'_url'], 'name':question.image[docname+'_name'], 'thumb':'/static/display/tn_'+question.image[docname+'_name']}})
            context.update({'q': question_id, 'question':question})

    # Load documents for the list page
    # documents = [{'docfile':{'url':'blaaa', 'name':'Bleee'}}]

    context.update( {'documents': documents, 'form': form,})
    context.update(csrf(request))
    context.update({'the_user': request.user})

    return direct_to_template(request, 'question/question_media.html', context)


def _change_image_size(fname_in, fname_out, basewidth=300):
    import PIL
    from PIL import Image

    img = Image.open(fname_in)
    # original_size = img,size[0]
    wpercent = (basewidth/float(img.size[0]))
    hsize = int((float(img.size[1])*float(wpercent)))
    img = img.resize((basewidth,hsize), PIL.Image.ANTIALIAS)
    img.save(fname_out)
    new_size = img.size

    # return {'original_size':original_size, 'new_size':new_size, 'percent':wpercent}


def _decode_image_to_size(file_data, orig_fname, new_fname, basewidth):
    import os
    if os.path.exists(new_fname):
        return
    file_decoded_data = base64.b64decode(file_data)
    f = file(orig_fname, 'w')
    f.write(file_decoded_data)
    f.close()
    _change_image_size(orig_fname, new_fname, basewidth=basewidth)


@login_required
def review_question_media(request):
    question_id = request.GET['q']
    question = Question.objects.get(id=question_id)

    documents = {}
    for docname in question.docs.keys():
        file_data = question.image[docname+'_data']
        file_name = question.image[docname+'_name']
        orig_fname = settings.STATIC_ROOT + '/../moma_example/static/display/' + file_name
        new_fname = settings.STATIC_ROOT + '/../moma_example/static/display/s_' + file_name
        _decode_image_to_size(file_data, orig_fname, new_fname, basewidth=300)
        documents.update({file_name:'/static/display/s_'+file_name})

    context = {}
    context.update( {'documents': documents, })
    context.update(csrf(request))
    context.update({'the_user': request.user})
    context.update({'question': question})

    return direct_to_template(request, 'question/review_question_media.html', context)


@login_required
def vote(request):
    allowed_votes = 3
    user = request.user

    already_voted=0
    questions = Question.objects.all()
    for question in questions:
        if user.id in question.vote_ids:
            already_voted += 1
        if already_voted >= 3:
            return render_to_response('question/home.html', {'questions':questions, 'the_user': request.user, 'message': 'You can only vote %s times!' % allowed_votes, 'msg_err':True})

    question_id = request.GET['q']
    question = Question.objects.get(id=question_id)
    if user.id not in question.vote_ids:
        question.vote_ids.append(user.id)
        question.save()
    return render_to_response('question/home.html', {'questions':Question.objects.all(), 'the_user': request.user, 'message': 'Voted!!' })



@login_required
def un_vote(request):
    user = request.user
    question_id = request.GET['q']
    question = Question.objects.get(id=question_id)
    if user.id in question.vote_ids:
        question.vote_ids.remove(user.id)
        question.save()
    return render_to_response('question/home.html', {'questions':Question.objects.all(), 'the_user': request.user, 'message': 'Un-voted!!' })


@login_required
def about(request):
    user = request.user
    context = {'the_user': request.user}
    context.update(csrf(request))
    return render_to_response('about.html', context)




DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3', # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': '/home/lucidel/projects/moma-django/moma_example/db.sqlite',  # Or path to database file if using sqlite3.
        'USER': '',                      # Not used with sqlite3.
        'PASSWORD': '',                  # Not used with sqlite3.
        'HOST': '',                      # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',                      # Set to empty string for default. Not used with sqlite3.
    }
}



#==========================================================================
# Copyright 2012 Lucidel, Inc., 2013 Cloudoscope Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================

from django.contrib.auth.models import User
from django.db import models
from moma_django.models import MongoModel, post_syncdb_mongo_handler
from moma_django.fields import MongoDateTimeField, DictionaryField, ValuesField


class Profile(User):
    company_name = models.CharField(max_length=200)

    @property
    def accounts(self):
        if self.is_superuser:
            return AnalyticsAccount.objects.all()
        else:
            return self.analyticsaccount_set.all()



class AnalyticsAccount(models.Model):
    user = models.ForeignKey(Profile)
    name = models.CharField(max_length=500)

    website_url = models.CharField(max_length=1000, null=True)


    created_on = models.DateTimeField(null=True, help_text='Time registered', blank=True, default=None)

    class Meta:
        ordering = ['name',]



# Enabling South for the non conventional mongo model
#
# add_introspection_rules(
#     [
#         (
#             (MongoIdField, MongoDateTimeField, DictionaryField ),
#             [],
#             {
#                 "max_length":   ["max_length",   {"default": None}],
#                 },
#         ),
#         ],
#     ["^mongo_django.fields.*",])



class UniqueVisit(MongoModel):
    account = models.ForeignKey('testing.AnalyticsAccount')
    date = MongoDateTimeField(db_index=True)
    visit_count = models.IntegerField()
    source = models.CharField(max_length=64 )
    referral_path = models.CharField(max_length=256 )
    keyword = models.CharField(max_length=256 )
    landing_page_path = models.CharField(max_length=256 )
    exit_page_path = models.CharField(max_length=256 )
    demographics_and_id = models.CharField(max_length=64 )
    time_on_site = models.FloatField(max_length=64 )
    page_views = models.IntegerField()
    campaign = models.CharField(max_length=64 )
    visit_id = models.CharField(max_length=64 )

    location = DictionaryField(item_field=models.CharField(max_length=64))
    demographics = DictionaryField(models.CharField(max_length=64))

    user_id = models.IntegerField()
    first_visit_date = MongoDateTimeField()

    goal_values = DictionaryField(models.FloatField())
    goal_starts = DictionaryField(models.IntegerField())
    goal_completions = DictionaryField(models.IntegerField())


    def __unicode__(self):
        return u'%s[%s %s %s]' % (self.account, self.date, self.visit_count, self.demographics_and_id)

    class Meta:
        unique_together = ['account', 'date', 'visit_count', 'visit_id']
        managed = False



#---------------------------------------
# TEST MODELS
# The following models are used only for testing purposes

class TestModel1(MongoModel):
    start_date = MongoDateTimeField()
    values = ValuesField(models.FloatField())
    some_id = models.IntegerField()

    def __unicode__(self):
        return u'TM1 %s[%s]' % (self.some_id, self.start_date)

    class Meta:
        unique_together = ['start_date', 'some_id', ]
        managed = False

class TestModel2(MongoModel):
    ref1 = models.ForeignKey('testing.TestModel1')
    start_date = MongoDateTimeField()
    values = ValuesField(models.FloatField())
    some_id = models.IntegerField()

    def __unicode__(self):
        return u'TM2 %s[%s]' % (self.some_id, self.start_date)

    class Meta:
        unique_together = ['start_date', 'some_id', ]
        managed = False

class TestModel3(MongoModel):
    ref1 = models.ForeignKey('testing.TestModel1')
    ref2 = models.ForeignKey('testing.TestModel2')
    start_date = MongoDateTimeField()
    values = ValuesField(models.FloatField())
    some_id = models.IntegerField()

    def __unicode__(self):
        return u'TM2 %s[%s]' % (self.some_id, self.start_date)

    class Meta:
        unique_together = ['start_date', 'some_id', ]
        managed = False


class TstBook(MongoModel):
    name = models.CharField(max_length=64)
    publish_date = MongoDateTimeField()
    author = models.ForeignKey('testing.TstAuthor')

    class Meta:
        unique_together = ['name', 'author', ]

    def __unicode__(self):
        au = TstAuthor.objects.get(id=self.author_id)
        return u'Book %s, by %s %s' % (self.name, au.first_name, au.last_name,)


class TstAuthor(MongoModel):
    first_name = models.CharField(max_length=32)
    last_name = models.CharField(max_length=32)




models.signals.post_syncdb.connect(post_syncdb_mongo_handler)

# End: TEST MODELS
#---------------------------------------



#==========================================================================
# Copyright 2012 Lucidel, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================

from django.db import models
from .queryset import MongoQuerySet

class MongoManager(models.Manager):
    
    def contribute_to_class(self, model, name):
        super(MongoManager, self).contribute_to_class(model, name)
        setattr(model, '_base_manager', getattr(model, '_default_manager'))
        self.using_collection_prefix = None

    def bulk_insert(self, items):
        #n = 10000
        
        #for i in xrange(0, len(items), n):
        #    self.get_query_set().insert(items[i:i+n])
        return self.get_query_set().insert(items)
    
    def _insert(self, values, fields, **kwargs):
        doc = {}
        for field in fields:
            for val in values:
                value = getattr(val, field.attname)
                doc[field.name] = value


        id = self.get_query_set().insert(doc)

        if kwargs.get('return_id'):
            return id
    
    def get_query_set(self):
        res = MongoQuerySet(self.model)
        if not hasattr(self,'using_collection_prefix'):
            self.using_collection_prefix = None
        if self.using_collection_prefix is not None:
            res._using_collection_prefix(self.using_collection_prefix)
        return res
    
    def ensure_index(self, *args, **kwargs):
        return self.get_query_set().ensure_index(*args, **kwargs)
    
    def index_information(self):
        return self.get_query_set().index_information()
    
    def reindex(self):
        return self.get_query_set().reindex()
    
    def values_list(self, *args, **kwargs):
        return self.get_query_set().values_list(*args, **kwargs)
    
    def drop_indexes(self):
        return self.get_query_set().drop_indexes()

    def _setTargetCollectionPrefix(self, using_collection_prefix=None):
        """
        This method enables redirecting the entire model onto another collection.
        Note: this should be used VERY carefully as it will redirect all reading and writing.
        Use using_collection_prefix=None to cancel the redirection.
        E.g. if using_collection_prefix='acme_' and the db name is 'customers', a new db will be used, called 'acme_customers'.
        """
        self.using_collection_prefix = using_collection_prefix



#==========================================================================
# Copyright 2012 Lucidel, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================

from copy import deepcopy
from django.utils.tree import Node
from .query_utils import fix_value
import pymongo
from pymongo.errors import AutoReconnect
from .exceptions import MongoQuerySetExeption
from django.conf import settings
from .query_utils import MongoQ, negate_where_clause

HOST = getattr(settings, 'MONGO_HOST', 'localhost')
PORT = getattr(settings, 'MONGO_PORT', 27017)
COLLECTION = getattr(settings, 'MONGO_COLLECTION')

try:
    connection = pymongo.Connection(HOST, PORT)
except (pymongo.errors.AutoReconnect, pymongo.errors.ConfigurationError):
    connection = None

class FakeCollection(object):

    def find(self, *args, **kwargs):
        return []
    
    def __getattr__(self, name):
        def fake_method(*args, **kwargs):
            return
        return fake_method

def get_collection(model, use_collection_name_prefix=None, override_default_db_name=False):
    collection_name = COLLECTION
    try:
        from django.db import connections
        default_connection = connections['default']
        default_db_name = default_connection.settings_dict['NAME']
        # the 'test_' is for a full sql db (e.g. mysql). The ':memory:' is for testing situation with sqlite as the main db
        if (not override_default_db_name) and default_db_name == ':memory:':
            collection_name = 'test_'+collection_name
        elif (not override_default_db_name) and default_db_name.startswith('test_'):
            collection_name = default_db_name
        if use_collection_name_prefix is not None:
            collection_name = use_collection_name_prefix + collection_name
    except Exception:
        pass

    if not connection:
        return FakeCollection()
    
    # NOTE - the following command ensures that the mongo will use a different database name once running in a unit test mode
    # usually, the default DB name is prepended with "test_" for the default SQL db. The MongoDB will follow. This ensures
    # that in unit-test situation, the database will change without affecting the original database
    db = connection[collection_name]

    opt = model._meta
    name = '%s_%s' % (opt.app_label, opt.object_name.lower())
    return db[name]

class MongoQuery(object):
    AND = 'AND'
    OR = 'OR'
    COLON = ':'
    DEFAULT = 'DEFAULT'

    def __init__(self, model):
        self.model = model
        self.where = Node()
        self.order_by = []
        self.fields = set()
        self.default_ordering = True
        self.select_related = True #Hack for admin-interface: django/contrib/admin/views/main.py line 210
        self._field_names = {}
        self.target_collection_prefix=None

    def _setTargetCollectionPrefix(self, prefix_str=None):
        """
        This method enables redirecting the entire model onto another collection.
        Note: this should be used VERY carefully as it will redirect all reading and writing.
        Use prefix_str=None to cancel the redirection.
        E.g. if prefix_str='acme_' and the db name is 'customers', a new db will be used, called 'acme_customers'.
        """
        self.target_collection_prefix=prefix_str

    def _column_name(self, field_name):
        if isinstance(field_name, (list, tuple)):
            return [self._column_name(f) for f in field_name]
        
        if not field_name in self._field_names:
            opt = self.model._meta

            if field_name == 'pk':
                field = opt.pk
            else:
                field = opt.get_field(field_name)
            
            self._field_names[field_name] = field.get_attname_column()[1]
        
        return self._field_names[field_name]

    def clear_ordering(self, force_empty=False):
        self.order_by = []
        if force_empty:
            self.default_ordering = False
        
    def add_ordering(self, ordering):
        if ordering:
            self.order_by.extend(ordering)
        else:
            self.default_ordering = False
        
    def can_filter(self):
        """
        For SQL returns False is result already fetched
        """
        return True
    
    def set_fields(self, fields):
        self.fields.update(set(fields))
    
    def get_fields(self):
        return list(self.fields)
    
    def clone(self):
        obj = self.__class__(self.model)
        obj.where = Node()
        obj.where = deepcopy(self.where)
        obj.order_by = deepcopy(self.order_by)
        obj.fields = deepcopy(self.fields)
        obj.target_collection_prefix = self.target_collection_prefix
        return obj
    
    def add_q(self, q_object):
        """
        Adds a Q-object to the current filter.

        Can also be used to add anything that has an 'add_to_query()' method.
        See also the method / property 'spec', that operates on the tree structure created in this method,
            kept in self.where
        """
        if hasattr(q_object, 'add_to_query'):
            # Complex custom objects are responsible for adding themselves.
            q_object.add_to_query(self)
        else:
            if self.where and q_object.connector != self.AND and len(q_object) > 1:
                self.where.start_subtree(self.AND)
                subtree = True
            else:
                subtree = False
            connector = self.AND
            for child in q_object.children:
                self.where.start_subtree(connector)
                if isinstance(child, Node):
                    self.add_q(child)
                else:
                    mq = MongoQ(False, child[0],child[1])
                    self.where.add(mq,self.COLON)
                self.where.end_subtree()
                connector = q_object.connector
            if q_object.negated:
                self.where.negate()
            if subtree:
                self.where.end_subtree()

    def get_count(self):
        return self.get_result().count()
    
    def has_results(self):
        return bool(self.get_count())
    
    def get_result(self):
        cursor = self.collection.find(self.spec, fields=self.fields and self._column_name(list(self.fields)) or None)

        sort = self.sort
        
        if sort:
            cursor = cursor.sort(sort)
        
        return cursor
    
    def update(self, values):
        data = {}
        for key, val in values.items():
            data[self._column_name(key)] = fix_value(val)

        self.collection.update(self.spec, {"$set": data})
    
    def delete(self):
        self.collection.remove(self.spec)
    
    @property
    def sort(self):
        output = []
        
        default_ordering = self.model._meta.ordering
        
        if not self.order_by and self.default_ordering and default_ordering:
            order_by = default_ordering
        else:
            order_by = self.order_by
        
        for item in order_by:
            if item.startswith('-'):
                output.append((self._column_name(item[1:]), pymongo.DESCENDING))
            else:
                output.append((self._column_name(item), pymongo.ASCENDING))
                
        return output

    def _process_where_node(self, oq, spec):
        if oq.connector != self.DEFAULT:
            if oq.connector == self.COLON:
                q = oq.children[0]
                column_orig = self._column_name(q._get_field_first_part())
                column = q.getDotNotationFieldName(self.model)
                qspec = q.spec(self.model)

                if not column in spec:
                    spec[column] = qspec
                else:
                    filter = spec[column]
                    if not isinstance(spec[column], dict) or not isinstance(qspec, dict):
                        raise MongoQuerySetExeption('Does not support = and <,>,=<,=> in one query (column:%s)' % column)
                    spec[column].update(qspec)
            elif oq.connector == self.AND:
                r = {}
                spec1 = {}
                for a_item in oq.children:
                    if a_item.connector == self.DEFAULT and len(a_item.children) == 1:
                        a_item = a_item.children[0]
                    if a_item.connector != self.DEFAULT:
                        r.update(self._process_where_node(a_item,spec1))
                spec = r
                if oq.negated:
                    spec = negate_where_clause(spec)
            elif oq.connector == self.OR:
                r = []
                for a_item in oq.children:
                    if a_item.connector != self.DEFAULT:
                        spec1 = {}
                        r.append(self._process_where_node(a_item,spec1))
                spec = self._or_operation_optimization({'$or': r})
                if oq.negated:
                    spec = negate_where_clause(spec)
        return spec


    def _or_operation_optimization(self, spec):
        """
        This method creates a very simple optimization to the case where there is a complete $or where statement,
        where all the items are in a form that can be replaced with in $in statement.
        For example, the following can be optimized:
          {'$or': [{'time_on_site': 10.0}, {'time_on_site': 25.0}, {'time_on_site': 275.0}, {'time_on_site': 148.0}]}
        However, the following CAN NOT be optimized:
          {'$or': [{'time_on_site': 10.0}, {'time_on_site': 25.0}, {'time_on_site': 275.0}, {'visit_count': 12.0}]}
          (because not all parameters are identical)
        Returns the original spec if can't be optimized, or a new $in where statement in the format:
          { x : { $in : [ a, b ] } }
        See also http://www.mongodb.org/display/DOCS/OR+operations+in+query+expressions for more information
        """
        param = None
        values = []
        for item in spec['$or']:
            p = item.keys()
            if len(p)!=1:
                return spec
            if param is None:
                param = p[0]
            if param != p[0]:
                return spec
            if isinstance(item[param], dict):
                return spec
            values.append(item[param])

        res = {param: {'$in': values}}
        return res


    @property
    def spec(self):
        if not self.where:
            return None
        
        spec = {}
        
#        for oq in self.where.children:
#            spec = self._process_where_node(oq, spec)
        spec = self._process_where_node(self.where, spec)

        return spec
    
    @property
    def collection(self):
        return get_collection(self.model,self.target_collection_prefix)
    
    def _prepare_before_insert(self, doc):
        if isinstance(doc, (list, tuple)):
            return [self._prepare_before_insert(d) for d in doc]
        
        data = {}
        
        for name, value in doc.items():
            data[self._column_name(name)] = value

        return data
    
    def insert(self, docs):
        return self.collection.insert(self._prepare_before_insert(docs), safe=True)



#==========================================================================
# Copyright 2012 Lucidel, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================

from django.db import models, router
from django.db.models.deletion import Collector
from .manager import MongoManager
from .fields import MongoIdField
import pymongo

class MongoModel(models.Model):
    id = MongoIdField()
    
    objects = MongoManager()
    
    class Meta:
        abstract = True
        managed = False #don't forget this to prevent Django create table in DB
        
    def delete(self, using=None):
        using = using or router.db_for_write(self.__class__, instance=self)
        assert self._get_pk_val() is not None, "%s object can't be deleted because its %s attribute is set to None." % (self._meta.object_name, self._meta.pk.attname)

        # Find all the objects than need to be deleted.
        collector = Collector(using=using)
        collector.collect([self])

        #hack to prevent ORM delet object via sql
        #it does not use QuerySet - directly sql module :(
        to_delete = {}
        for k in collector.data.keys()[:]:
            if issubclass(k,MongoModel):
                to_delete.update({k:collector.data.pop(k)})

        for key, object_set in to_delete.items():
            for obj in object_set:
                cls = obj.__class__
                if not cls._meta.auto_created:
                    models.signals.pre_delete.send(sender=cls, instance=obj)

                cls._default_manager.filter(pk=obj.pk).delete()

                if not cls._meta.auto_created:
                    models.signals.post_delete.send(sender=cls, instance=obj)


        # Delete other objects.
        collector.delete()
#        delete_objects(seen_objs, using)

    delete.alters_data = True
    
    @classmethod
    def ensure_indexes(cls, verbosity=1):
        #FIXME: add supporind mongo index features like asceding indexing
        opt = cls._meta

        if cls._requires_indexing(verbosity) is False:
            if verbosity >= 1:
                print u'Skipping index installation for %s model (indexes already aligned with the model).' % opt
            return

        if verbosity >= 1:
            print u'Installing index for %s model' % opt

        cls.objects.drop_indexes()

        for unique_fields in opt.unique_together:
            if verbosity >= 2:
                print u'Ensure uniquer indexes for %s' % opt, unique_fields
            
            args = []
            
            for f in unique_fields:
                column = opt.get_field(f).get_attname_column()[1]
                args.append((column, pymongo.DESCENDING))

            cls.objects.ensure_index(args, unique=True, dropDups=True, name=('%s_%s' % (cls.__name__,'unique_together')))
        
        for field in opt.fields:
            if field.db_index:
                column = field.get_attname_column()[1]
                if field.unique:
                    if verbosity >= 2:
                        print 'Ensure unique index for', cls.__name__, field.name
                    cls.objects.ensure_index([(column, pymongo.DESCENDING)], unique=True, dropDups=True)
                else:
                    if verbosity >= 2:
                        print 'Ensure index for', cls.__name__, field.name                    
                    cls.objects.ensure_index([(column, pymongo.DESCENDING)])

    @classmethod
    def _requires_indexing(cls, verbosity=1):
        prevIndexes = cls.objects.index_information()
        newIndexes = {}

        key = u'_id_'
        item = {u'key':[(u'_id',1)], u'v':1}
        newIndexes.update({key:item})

        opt = cls._meta
        unique_together_index_name=('%s_%s' % (cls.__name__,'unique_together'))

        for unique_fields in opt.unique_together:
            if verbosity >= 2:
                print u'Ensure uniquer indexes for %s' % opt, unique_fields

            args = []

            for f in unique_fields:
                column = opt.get_field(f).get_attname_column()[1]
                args.append((u'%s'%column, pymongo.DESCENDING))

            item = {u'dropDups':True, u'key':args, u'unique':True, u'v':1}
            newIndexes.update({unique_together_index_name:item})


        for field in opt.fields:
            if field.db_index:
                column = field.get_attname_column()[1]
                if field.unique:
                    if verbosity >= 2:
                        print 'Ensure unique index for', cls.__name__, field.name
                    key = u'%s_%s' % (column, pymongo.DESCENDING)
                    item = {u'dropDups':True, u'key':[(u'%s'%column, pymongo.DESCENDING)], u'unique':True, u'v':1}
                    newIndexes.update({key:item})
                else:
                    if verbosity >= 2:
                        print 'Ensure index for', cls.__name__, field.name
                    key = u'%s_%s' % (column, pymongo.DESCENDING)
                    item = {u'key':[(u'%s'%column, pymongo.DESCENDING)], u'v':1}
                    newIndexes.update({key:item})

        if prevIndexes == newIndexes:
            return False
        return True



def post_syncdb_mongo_handler(**kwargs):
    """
    This handler should be added once if you have indexes in MongoModels:
    
    `models.signals.post_syncdb.connect(post_syncdb_mongo_handler)`
    """
    app = kwargs.get('app')
    for model in models.get_models(app):
        if issubclass(model, MongoModel):
            model.ensure_indexes(kwargs.get('verbosity'))



#==========================================================================
# Copyright 2012 Lucidel, Inc., 2013 Cloudoscope Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================

import data
from django.conf.urls import patterns, include, url
from django.contrib import admin
from moma_example import settings

admin.autodiscover()

urlpatterns = patterns('',
    url(r'^$', 'moma_example.views.home', name='home'),

    url(r'^admin/', include(admin.site.urls)),
    (r'^media/(?P<path>.*)$', 'django.views.static.serve', {'document_root': settings.MEDIA_ROOT}),
    (r'^static/(?P<path>.*)$', 'django.views.static.serve', {'document_root': settings.STATIC_ROOT}),

    # User auth urls
    url(r'^accounts/login/$', 'moma_example.views.login', name='login'),
    url(r'^accounts/auth/$', 'moma_example.views.auth_view'),
    url(r'^accounts/logout/$', 'moma_example.views.logout', name='logout'),
    url(r'^accounts/loggedin/$', 'moma_example.views.loggedin'),
    url(r'^accounts/invalid/$', 'moma_example.views.invalid_login'),
    url(r'^accounts/register/$', 'moma_example.views.register_user', name='register'),
    url(r'^accounts/register_success/$', 'moma_example.views.register_success'),

    url(r'^about/$', 'data.views.about', name='about'),

    url(r'^q/home/$', 'data.views.questions_home', name='questions_home'),
    url(r'^q/edit/$', 'data.views.questions_edit'),
    url(r'^q/vote/$', 'data.views.vote'),
    url(r'^q/un_vote/$', 'data.views.un_vote'),
    url(r'^q/media/$', 'data.views.list_question_media', name='list_question_media'),
    url(r'^q/review/$', 'data.views.review_question_media', name='review_question_media'),
    url(r'^q/search/$', 'data.views.questions_search', name='questions_search'),
)



#==========================================================================
# Copyright 2012 Lucidel, Inc., 2013 Cloudoscope Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================

"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""

from django.test import TestCase


class SimpleTest(TestCase):
    def test_basic_addition(self):
        """
        Tests that 1 + 1 always equals 2.
        """
        self.assertEqual(1 + 1, 2)



#==========================================================================
# Copyright 2012 Lucidel, Inc., 2013 Cloudoscope Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================
from django.db.models import Q
from django.test import TestCase
from models import Profile, AnalyticsAccount, TestModel1, TestModel2, TestModel3, UniqueVisit
from datetime import datetime
from django.utils import timezone

class TestMongoQuerySetWithUniqueVisits(TestCase):

    def setUp(self):
        self.user = Profile()
        self.user.username = 'testuser'
        self.user.company_name = 'Test Company'
        self.user.set_password('testuser')
        self.user.save()

        self.acc = AnalyticsAccount(user = self.user)
        self.acc.pk = 10000
        self.acc.name = 'Test'
        self.acc.analytics_id = 'asd324sf456m6n76b5b'
        self.acc.history_fetched = False
        self.acc.save()

        self.other_acc = AnalyticsAccount(user = self.user)
        self.other_acc.pk = 10001
        self.other_acc.name = 'Test1'
        self.other_acc.analytics_id = 'asd324sf456m111111'
        self.other_acc.history_fetched = False
        self.other_acc.save()

        self.list_of_accounts = [self.acc, self.other_acc]

        records = []


        class NumberLong():
            def __init__(self, num):
                self.n = num


        records.append({ "goal_starts" : { }, "time_on_site" : 17, "user_id" : 650825334, "account_id" : NumberLong(5),
                         "campaign" : "(not set)", "location" : { "cr" : "United States", "rg" : "Tennessee",
                                                                  "ct" : "Clarksville" }, "demographics" :
                             { "age" : "GenX (25-44)", "education" : "High School" }, "first_visit_date" :
                             ISODate("2012-08-04T21:18:17Z"), "referral_path" : "(not set)", "source" : "google", "exit_page_path" :
                             "/some-analysis/g1iar-daisy/", "landing_page_path" : "(not set)", "keyword" :
                             "g1iar daisy sparknotes", "date" : ISODate("2012-08-04T00:00:00Z"), "goal_completions" : { },
                         "visit_count" : 1, "visit_id" : "0---30----0---------------650825334.1344115097",
                         "page_views" : 3, "goal_values" : { } })
        records.append({ "goal_starts" : { }, "time_on_site" : 36, "user_id" : 277227593, "account_id" : NumberLong(5),
                         "campaign" : "(not set)", "location" : { "cr" : "United States", "rg" : "Tennessee",
                                                                  "ct" : "Sevierville" }, "demographics" :
                             { "gender" : "Men", "education" : "High School" }, "first_visit_date" :
                             ISODate("2012-08-06T14:46:23Z"), "referral_path" : "(not set)", "source" : "google", "exit_page_path" :
                             "/some-analysis/looking-at-my-ancestors/", "landing_page_path" : "(not set)", "keyword" :
                             "(not provided)", "date" : ISODate("2012-08-06T00:00:00Z"), "goal_completions" : { }, "visit_count" : 1,
                         "visit_id" : "0-2--0-1---1---------------277227593.1344264383", "page_views" : 1,
                         "goal_values" : { } })
        records.append({ "goal_starts" : { }, "time_on_site" : 27, "user_id" : 1429730596, "account_id" : NumberLong(5),
                         "campaign" : "(not set)", "location" : { "cr" : "United States", "rg" : "Virginia",
                                                                  "ct" : "Burke" }, "demographics" :
                             { "gender" : "Men", "age" : "Baby Boomers (45-70)", "education" : "High School" },
                         "first_visit_date" : ISODate("2012-08-06T04:28:22Z"), "referral_path" : "(not set)", "source" :
            "google", "exit_page_path" : "/some-analysis/the_double-blind/", "landing_page_path" : "(not set)",
                         "keyword" : "(not provided)", "date" : ISODate("2012-08-06T00:00:00Z"), "goal_completions" :
                    { }, "visit_count" : 1, "visit_id" : "0-2-90-1--0---------------1429730596.1344227302",
                         "page_views" : 3, "goal_values" : { } })
        records.append({ "goal_starts" : { }, "time_on_site" : 27, "user_id" : 2110905334, "account_id" : NumberLong(5),
                         "campaign" : "(not set)", "location" : { "cr" : "United States", "rg" : "New York",
                                                                  "ct" : "Poughkeepsie" }, "demographics" :
                             { "gender" : "Men", "education" : "High School", "political" : "Democrats" }, "first_visit_date" :
                             ISODate("2012-08-06T18:21:11Z"), "referral_path" : "(not set)", "source" : "yahoo", "exit_page_path" :
                             "/some-analysis/joe-and-bobby/", "landing_page_path" : "(not set)", "keyword" :
                             "book summaries", "date" : ISODate("2012-08-06T00:00:00Z"), "goal_completions" : { },
                         "visit_count" : 1, "visit_id" : "0-0--0-1---------------2110905334.1344277271",
                         "page_views" : 1, "goal_values" : { } })
        records.append({ "goal_starts" : { }, "time_on_site" : 120, "user_id" : 952676001, "account_id" : NumberLong(5),
                         "campaign" : "(not set)", "location" : { "cr" : "United States", "rg" : "Massachusetts",
                                                                  "ct" : "Abington" }, "demographics" :
                             { "age" : "GenX (25-44)", "education" : "High School" },
                         "first_visit_date" : ISODate("2012-08-06T05:53:52Z"), "referral_path" : "(not set)",
                         "source" : "google", "exit_page_path" : "/some-analysis/the-blue-bush/",
                         "landing_page_path" : "(not set)", "keyword" : "(not provided)",
                         "date" : ISODate("2012-08-06T00:00:00Z"), "goal_completions" : { }, "visit_count" : 1,
                         "visit_id" : "0---30---------------952676001.1344232432", "page_views" : 3,
                         "goal_values" : { } })
        records.append({ "goal_starts" : { }, "time_on_site" : 10, "user_id" : 805172613, "account_id" : NumberLong(5),
                         "campaign" : "(not set)", "location" : { "cr" : "United States", "rg" : "New York",
                                                                  "ct" : "Peekskill" }, "demographics" :
                             { "age" : "GenX (25-44)", "education" : "High School" }, "first_visit_date" :
                             ISODate("2012-08-06T16:27:55Z"), "referral_path" : "(not set)", "source" : "bing", "exit_page_path" :
                             "/wikipedia/rama-ii/", "landing_page_path" : "(not set)", "keyword" : "research rama ii", "date" :
                             ISODate("2012-08-06T00:00:00Z"), "goal_completions" : { }, "visit_count" : 2, "visit_id" :
                             "0---30---------------805172613.1344270475", "page_views" : 2, "goal_values" : { } })
        records.append({ "goal_starts" : { }, "time_on_site" : 147, "user_id" : 2060101123, "account_id" : NumberLong(5),
                         "campaign" : "(not set)", "location" : { "cr" : "United States", "rg" : "South Carolina", "ct" :
            "Charleston" }, "demographics" : { "age" : "GenX (25-44)", "education" : "High School" },
                         "first_visit_date" : ISODate("2012-08-06T02:04:05Z"), "referral_path" : "(not set)", "source" :
            "google", "exit_page_path" : "/search", "landing_page_path" : "(not set)", "keyword" :
                             "a short reference", "date" : ISODate("2012-08-06T00:00:00Z"), "goal_completions" : { },
                         "visit_count" : 1, "visit_id" : "0-2-30---------------2060101123.1344218645",
                         "page_views" : 3, "goal_values" : { } })
        records.append({ "goal_starts" : { }, "time_on_site" : 25, "user_id" : 1525231829, "account_id" : NumberLong(5),
                         "campaign" : "(not set)", "location" : { "cr" : "United States", "rg" : "New York", "ct" :
            "Massapequa" }, "demographics" : { "age" : "GenX (25-44)", "education" : "High School" },
                         "first_visit_date" : ISODate("2012-08-05T14:57:17Z"), "referral_path" : "(not set)", "source" :
            "google", "exit_page_path" : "/wikipedia/indian-reservations/", "landing_page_path" : "(not set)", "keyword" :
                             "indian reservations in north america", "date" : ISODate("2012-08-08T00:00:00Z"), "goal_completions" : { },
                         "visit_count" : 2, "visit_id" : "0---30---------------1525231829.1344178637",
                         "page_views" : 2, "goal_values" : { } })
        records.append({ "_id" : ObjectId("502abdabf7f16836f1002847"), "goal_starts" : { }, "time_on_site" : 32,
                         "user_id" : 1322541271, "account_id" : NumberLong(5), "campaign" : "(not set)", "location" :
            { "cr" : "United States", "rg" : "Ohio", "ct" : "Amherst" }, "demographics" : { },
                         "first_visit_date" : ISODate("2012-07-31T12:33:33Z"), "referral_path" : "(not set)", "source" :
            "google", "exit_page_path" : "/some-analysis/the-language-barrier/", "landing_page_path" :
                             "(not set)", "keyword" : "the language barrier", "date" : ISODate("2012-07-31T00:00:00Z"),
                         "goal_completions" : { }, "visit_count" : 1, "visit_id" :
            "false---------------1322541271.1343738013", "page_views" : 2, "goal_values" : { } })
        records.append({ "_id" : ObjectId("502abdabf7f16836f1002848"), "goal_starts" : { }, "time_on_site" : 26,
                         "user_id" : 1044048896, "account_id" : NumberLong(5), "campaign" : "(not set)", "location" :
            { "cr" : "United States", "rg" : "New York", "ct" : "New York" }, "demographics" : { },
                         "first_visit_date" : ISODate("2012-07-31T16:42:23Z"), "referral_path" : "(not set)", "source" :
            "google", "exit_page_path" : "/some-analysis/mercy/", "landing_page_path" : 12, "keyword" :
                             "mercy discussion", "date" : ISODate("2012-07-31T00:00:00Z"), "goal_completions" : { }, "visit_count" :
                1, "visit_id" : "false---------------1044048896.1343752943", "page_views" : 1, "goal_values" :
                             { } })
        records.append({ "_id" : ObjectId("502abdabf7f16836f100284b"), "goal_starts" : { }, "time_on_site" : 38,
                         "user_id" : 184868780, "account_id" : NumberLong(5), "campaign" : "(not set)", "location" :
            { "cr" : "Ireland", "rg" : "Dublin", "ct" : "Dublin" }, "demographics" : { }, "first_visit_date" :
                             ISODate("2012-07-30T14:47:47Z"), "referral_path" : "(not set)", "source" : "google", "exit_page_path" :
                             "/some-analysis/jewelery-cutting/", "landing_page_path" : "(not set)", "keyword" :
                             "practical jewel setting and cutting", "date" : ISODate("2012-07-30T00:00:00Z"), "goal_completions" :
                    { }, "visit_count" : 1, "visit_id" : "false---------------184868780.1343659667",
                         "page_views" : 3, "goal_values" : { } })
        records.append({ "_id" : ObjectId("502abdabf7f16836f100284c"), "goal_starts" : { }, "time_on_site" : 196,
                         "user_id" : 1805411236, "account_id" : NumberLong(5), "campaign" : "(not set)", "location" :
            { "cr" : "India", "rg" : "Kerala", "ct" : "Cochin" }, "demographics" : { }, "first_visit_date" :
                             ISODate("2012-07-30T12:29:05Z"), "referral_path" : "/analysis-nama-ecological-expert/", "source" :
                             "articlemyriad.com", "exit_page_path" : "/some-analysis/the-financial-expert/", "landing_page_path" :
                             "/analysis-nama-ecological-expert/", "keyword" : "(not set)", "date" : ISODate("2012-07-30T00:00:00Z"),
                         "goal_completions" : { }, "visit_count" : 1, "visit_id" :
            "false---------------1805411236.1343651345", "page_views" : 11, "goal_values" : { } })
        records.append({ "_id" : ObjectId("502abdabf7f16836f100284d"), "goal_starts" : { }, "time_on_site" : 11,
                         "user_id" : 169596409, "account_id" : NumberLong(5), "campaign" : "(not set)", "location" :
            { "cr" : "United States", "rg" : "Pennsylvania", "ct" : "Willow Grove" }, "demographics" :
                             { "education" : "College" }, "first_visit_date" : ISODate("2012-07-31T12:16:26Z"), "referral_path" :
                             "(not set)", "source" : "yahoo", "exit_page_path" : "/some-analysis/the-maincoon-cat/", "landing_page_path" :
                             "(not set)", "keyword" : "the maincoon cat", "date" : ISODate("2012-07-31T00:00:00Z"),
                         "goal_completions" : { }, "visit_count" : 1, "visit_id" :
                "0----2---------------169596409.1343736986", "page_views" : 1, "goal_values" : { } })
        records.append({ "_id" : ObjectId("502abdabf7f16836f100284e"), "goal_starts" : { }, "time_on_site" : 275,
                         "user_id" : 157428922, "account_id" : NumberLong(5), "campaign" : "(not set)", "location" :
            { "cr" : "United States", "rg" : "Ohio", "ct" : "Chillicothe" }, "demographics" : { },
                         "first_visit_date" : ISODate("2012-08-01T00:20:25Z"), "referral_path" : "(not set)", "source" :
            "google", "exit_page_path" : "/some-analysis/a-bend-in-the-road/", "landing_page_path" :
                             21, "keyword" : "(not provided)", "date" : ISODate("2012-07-31T00:00:00Z"), "goal_completions" :
                    { }, "visit_count" : 1, "visit_id" : "false---------------157428922.1343780425",
                         "page_views" : 1, "goal_values" : { } })
        records.append({ "_id" : ObjectId("502abdabf7f16836f100284f"), "goal_starts" : { }, "time_on_site" : 148,
                         "user_id" : 1419573589, "account_id" : NumberLong(5), "campaign" : "(not set)", "location" :
            { "cr" : "South Africa", "rg" : "Western Cape", "ct" : "Cape Town" }, "demographics" : { },
                         "first_visit_date" : ISODate("2012-07-31T19:02:10Z"), "referral_path" : "(not set)", "source" :
            "google", "exit_page_path" : "/some-analysis/the-american-eagle/", "landing_page_path" :
                             "(not set)", "keyword" : "american eagle habitats", "date" : ISODate("2012-07-31T00:00:00Z"),
                         "goal_completions" : { }, "visit_count" : 1, "visit_id" :
                "false---------------1419573589.1343761330", "page_views" : 2, "goal_values" : { } })
        records.append({ "_id" : ObjectId("502abdabf7f16836f1002850"), "goal_starts" : { }, "time_on_site" : 54,
                         "user_id" : 553686581, "account_id" : NumberLong(5), "campaign" : "(not set)", "location" :
            { "cr" : "United States", "rg" : "California", "ct" : "Whittier" }, "demographics" :
                             { "age" : "GenX (25-44)", "education" : "College" }, "first_visit_date" :
                             ISODate("2012-07-31T17:39:12Z"), "referral_path" : "(not set)", "source" : "yahoo", "exit_page_path" :
                             "/search", "landing_page_path" : "(not set)", "keyword" : "book summaries", "date" :
                             ISODate("2012-07-31T00:00:00Z"), "goal_completions" : { }, "visit_count" : 1, "visit_id" :
                             "06--32----0---------------553686581.1343756352", "page_views" : 4, "goal_values" : { } })
        records.append({ "_id" : ObjectId("502abdabf7f16836f1002852"), "goal_starts" : { }, "time_on_site" : 105,
                         "user_id" : 804661250, "account_id" : NumberLong(5), "campaign" : "(not set)", "location" :
            { "cr" : "United States", "rg" : "Missouri", "ct" : "St Louis" }, "demographics" :
                             { "education" : "Grad School" }, "first_visit_date" : ISODate("2012-07-30T20:56:39Z"),
                         "referral_path" : "(not set)", "source" : "google", "exit_page_path" :
            "/some-analysis/people-of-australia/", "landing_page_path" : "(not set)", "keyword" :
                             "people of australia study guide", "date" : ISODate("2012-07-30T00:00:00Z"),
                         "goal_completions" : { }, "visit_count" : 1, "visit_id" :
            "0----3---------------804661250.1343681799", "page_views" : 2, "goal_values" : { } })
        records.append({ "_id" : ObjectId("502abdabf7f16836f1002857"), "goal_starts" : { }, "time_on_site" : 68,
                         "user_id" : 1587838983, "account_id" : NumberLong(5), "campaign" : "(not set)", "location" :
            { "cr" : "United States", "rg" : "California", "ct" : "San Francisco" }, "demographics" : { },
                         "first_visit_date" : ISODate("2012-07-30T07:55:27Z"), "referral_path" : "(not set)", "source" :
            "google", "exit_page_path" : "/wikipedia/riders-of-the-west/", "landing_page_path" : "(not set)",
                         "keyword" : "(not provided)", "date" : ISODate("2012-07-30T00:00:00Z"), "goal_completions" :
                    { }, "visit_count" : 1, "visit_id" : "false---------------1587838983.1343634927",
                         "page_views" : 2, "goal_values" : { } })
        records.append({ "_id" : ObjectId("502abdabf7f16836f1002859"), "goal_starts" : { }, "time_on_site" : 10,
                         "user_id" : 876838713, "account_id" : NumberLong(5), "campaign" : "(not set)", "location" :
            { "cr" : "United States", "rg" : "Michigan", "ct" : "Hart" }, "demographics" : { },
                         "first_visit_date" : ISODate("2012-07-30T11:47:30Z"), "referral_path" : "(not set)", "source" :
            "google", "exit_page_path" : "/some-analysis/the-cover-story/", "landing_page_path" :
                             "(not set)", "keyword" : "cover story notes", "date" : ISODate("2012-07-30T00:00:00Z"),
                         "goal_completions" : { }, "visit_count" : 1, "visit_id" :
            "false---------------876838713.1343648850", "page_views" : 1, "goal_values" : { } })
        records.append({ "_id" : ObjectId("502abdabf7f16836f100285a"), "goal_starts" : { }, "time_on_site" : 290,
                         "user_id" : 1154449631, "account_id" : NumberLong(5), "campaign" : "(not set)", "location" :
            { "cr" : "United States", "rg" : "California", "ct" : "Pasadena" }, "demographics" : { },
                         "first_visit_date" : ISODate("2012-07-30T17:10:06Z"), "referral_path" : "(not set)", "source" :
            "google", "exit_page_path" : "/some-analysis/lion-king/", "landing_page_path" : "(not set)",
                         "keyword" : "wikipedia lion king", "date" : ISODate("2012-07-30T00:00:00Z"),
                         "goal_completions" : { }, "visit_count" : 1, "visit_id" :
            "false---------------1154449631.1343668206", "page_views" : 3, "goal_values" : { } })

        self.the_records = records
        for i in self.the_records:
            i.pop('account_id')
            if '_id' in i:
                i.pop('_id')
            i['account'] = self.acc.pk

        UniqueVisit.objects.filter(account__in=self.list_of_accounts).delete()
        self.assertEqual(UniqueVisit.objects.filter(account__in=self.list_of_accounts).count(), 0)

        UniqueVisit.objects.bulk_insert(self.the_records)

        self.assertEqual(UniqueVisit.objects.filter(account__in=self.list_of_accounts).count(), len(self.the_records))


    def test_insert(self):
        """
        Taken from here https://github.com/django/django/blob/master/tests/modeltests/basic/tests.py
        """
        UniqueVisit.objects.filter(account__in=self.list_of_accounts).delete()
        self.assertEqual(UniqueVisit.objects.filter(account__in=self.list_of_accounts).count(), 0)

        UniqueVisit.objects.bulk_insert(self.the_records)

        self.assertEqual(UniqueVisit.objects.filter(account__in=self.list_of_accounts).count(), len(self.the_records))


    def test_simple_account_reference_query(self):
        qs = UniqueVisit.objects.filter(account = self.acc.pk)
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'account_id': 10000}
        ))
        for i in self.the_records:
            num += 1
        self.assertEqual(qs.count(), num)


    def test_simple_time_reference_query(self):
        qs = UniqueVisit.objects.filter(first_visit_date__lte =ISODate("2012-07-30T12:29:05Z"))
        num = 0
        self.assertEqual( qs.query.spec, dict({'first_visit_date': {'$lte': datetime(2012, 7, 30, 12, 29, 5, tzinfo=timezone.utc)}}))
        for i in self.the_records:
            if i['first_visit_date'] <= ISODate("2012-07-30T12:29:05Z"):
                num += 1
        self.assertEqual(qs.count(), num)


    def test_simple_time_reference_query_multiple_terms2(self):
        qs = UniqueVisit.objects.filter(first_visit_date__lte =ISODate("2012-07-30T12:29:05Z"),time_on_site__gt =10)
        num = 0
        self.assertEqual( qs.query.spec, dict({'time_on_site': {'$gt': 10.0}, 'first_visit_date':
            {'$lte': datetime(2012, 7, 30, 12, 29, 5, tzinfo=timezone.utc)}}))
        for i in self.the_records:
            if i['first_visit_date'] <= ISODate("2012-07-30T12:29:05Z") and i['time_on_site'] > 10:
                num += 1
        self.assertEqual(qs.count(), num)


    def test_simple_time_reference_query_multiple_terms3(self):
        qs = UniqueVisit.objects.filter(first_visit_date__lte =ISODate("2012-07-30T12:29:05Z"),time_on_site__gt =10,
                                        page_views__gt =2)
        num = 0
        self.assertEqual( qs.query.spec, dict({'time_on_site': {'$gt': 10.0}, 'page_views': {'$gt': 2},
                                               'first_visit_date': {'$lte': datetime(2012, 7, 30, 12, 29, 5, tzinfo=timezone.utc)}}))
        for i in self.the_records:
            if i['first_visit_date'] <= ISODate("2012-07-30T12:29:05Z") and \
                            i['time_on_site'] > 10 and \
                            i['page_views'] > 2:
                num += 1
        self.assertEqual(qs.count(), num)


    def test_simple_time_reference_query_with_Q(self):
        qs = UniqueVisit.objects.filter(Q(first_visit_date__lte =ISODate("2012-07-30T12:29:05Z")))
        num = 0
        self.assertEqual( qs.query.spec, dict({'first_visit_date': {'$lte': datetime(2012, 7, 30, 12, 29, 5, tzinfo=timezone.utc)}}))
        for i in self.the_records:
            if i['first_visit_date'] <= ISODate("2012-07-30T12:29:05Z"):
                num += 1
        self.assertEqual(qs.count(), num)


    def test_simple_time_reference_query_with_Q_2_AND(self):
        qs = UniqueVisit.objects.filter(Q(first_visit_date__lte =ISODate("2012-07-30T12:29:05Z"))&Q(time_on_site__gt =10))
        num = 0
        self.assertEqual( qs.query.spec, dict({'time_on_site': {'$gt':10.0}, 'first_visit_date':
            {'$lte': datetime(2012, 7, 30, 12, 29, 5, tzinfo=timezone.utc)}}))
        for i in self.the_records:
            if i['first_visit_date'] <= ISODate("2012-07-30T12:29:05Z") and i['time_on_site']>10:
                num += 1
        self.assertEqual(qs.count(), num)


    def test_simple_time_reference_query_with_Q_2_OR(self):
        qs = UniqueVisit.objects.filter(Q(first_visit_date =ISODate("2012-07-30T12:29:05Z"))|Q(time_on_site =10))
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'$or': [{'first_visit_date': datetime(2012, 7, 30, 12, 29, 5, tzinfo=timezone.utc)}, {'time_on_site': 10.0}]}
        ))
        for i in self.the_records:
            if i['first_visit_date']  ==  ISODate("2012-07-30T12:29:05Z") or i['time_on_site'] == 10:
                num += 1
        self.assertEqual(qs.count(), num)


    def test_simple_time_reference_query_with_Q_3_OR_with_same_param(self):
        qs = UniqueVisit.objects.filter(Q(time_on_site =10)|Q(time_on_site =25)|Q(time_on_site =275))
        num = 0
        self.assertEqual( qs.query.spec, dict(
            # old non optimized   {'$or': [{'time_on_site': 10.0}, {'time_on_site': 25.0}, {'time_on_site': 275.0}]}
            {'time_on_site': {'$in': [10.0, 25.0, 275.0]}}
        ))
        for i in self.the_records:
            if i['time_on_site'] == 25 or i['time_on_site'] == 10 or i['time_on_site'] == 275:
                num += 1
        qs = qs.filter(account__in=self.list_of_accounts)
        self.assertEqual(qs.count(), num)

    def test_query_with_in_opr(self):
        qs = UniqueVisit.objects.filter(time_on_site__in=[10, 25, 274])
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'time_on_site': {'$in': [10.0, 25.0, 274.0]}}
        ))
        for i in self.the_records:
            if i['time_on_site'] == 25 or i['time_on_site'] == 10 or i['time_on_site'] == 274:
                num += 1
        qs = qs.filter(account__in=self.list_of_accounts)
        self.assertEqual(qs.count(), num)

    def test_simple_time_reference_query_with_Q_4_OR_with_different_param(self):
        qs = UniqueVisit.objects.filter(Q(time_on_site =10)|Q(time_on_site =25)|Q(time_on_site =275)|Q(source = 'bing'))
        num = 0
        for i in self.the_records:
            if i['time_on_site']  ==  25 or i['time_on_site'] == 10 or i['time_on_site'] == 275:
                num += 1
        qs2 = qs.filter(account__in=self.list_of_accounts)
        self.assertEqual(qs2.count(), num)
        self.assertEqual( qs.query.spec, dict(
            {'$or': [{'time_on_site': 10.0}, {'time_on_site': 25.0}, {'time_on_site': 275.0}, {'source': 'bing'}]}
        ))

    def test_simple_time_reference_query_with_Q_4_OR_with_same_param_different_operation(self):
        qs = UniqueVisit.objects.filter(Q(time_on_site = 10)|Q(time_on_site = 25)|Q(time_on_site = 275) | Q(time_on_site__gt = 7))
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'$or': [{'time_on_site': 10.0}, {'time_on_site': 25.0}, {'time_on_site': 275.0},
                     {'time_on_site': {'$gt': 7.0}}]}
        ))
        for i in self.the_records:
            if i['time_on_site'] == 25 or i['time_on_site'] == 10 or i['time_on_site'] == 275 or i['time_on_site']>7:
                num += 1
        qs = qs.filter(account__in=self.list_of_accounts)
        self.assertEqual(qs.count(), num)

    def test_query_with_Q_AND_OR_combination(self):
        qs = UniqueVisit.objects.filter(Q(time_on_site =10)|Q(time_on_site =25)|Q(time_on_site =275)&Q(source = 'bing'))
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'$or': [{'time_on_site': 10.0}, {'time_on_site': 25.0}, {'time_on_site': 275.0, 'source': 'bing'}]}
        ))
        for i in self.the_records:
            if i['time_on_site'] == 25 or i['time_on_site'] == 10 or (i['time_on_site'] == 275 and i['source'] ==  'bing'):
                num += 1
        qs = qs.filter(account__in=self.list_of_accounts)
        self.assertEqual(qs.count(), num)


    def test_query_with_Q_OR_tree_combination(self):
        qs = UniqueVisit.objects.filter(Q(time_on_site =10)|(Q(time_on_site =25)|Q(time_on_site =275))|Q(time_on_site =148))
        num = 0
        self.assertEqual( qs.query.spec, dict(
            # old, non optimized:  {'$or': [{'time_on_site': 10.0}, {'time_on_site': 25.0}, {'time_on_site': 275.0}, {'time_on_site': 148.0}]}
            {'time_on_site': {'$in': [10.0, 25.0, 275.0, 148.0]}}
        ))
        for i in self.the_records:
            if i['time_on_site'] == 25 or i['time_on_site'] == 10 or (i['time_on_site'] == 275 or i['time_on_site'] == 148):
                num += 1
        qs = qs.filter(account__in=self.list_of_accounts)
        self.assertEqual(qs.count(), num)


    def test_simple_time_reference_query_with_Q_3_OR(self):
        qs = UniqueVisit.objects.filter(Q(time_on_site = 10) | Q(first_visit_date = ISODate("2012-07-30T12:29:05Z"))|
                                        Q(visit_count__lte = 2))
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'$or': [{'time_on_site': 10.0}, {'first_visit_date': datetime(2012, 7, 30, 12, 29, 5, tzinfo=timezone.utc)},
                     {'visit_count': {'$lte': 2}}]}
        ))
        for i in self.the_records:
            if i['time_on_site'] == 10 or i['first_visit_date'] == ISODate("2012-07-30T12:29:05Z") or i['visit_count']<=2:
                num += 1
        self.assertEqual(qs.count(), num)


    def test_simple_time_reference_query_with_Q_N_OR_regex_build(self):
        lst = [ 'facebook', 'yahoo', '.com']
        kwargs = {'source__regex': 'blog'}
        the_q = Q(**kwargs)
        for item in lst:
            kwargs = {'source__regex': item}
            the_q |= Q(**kwargs)
        qs = UniqueVisit.objects.filter(the_q)
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'$or': [{'source': {'$regex': 'blog'}}, {'source': {'$regex': 'facebook'}},
                     {'source': {'$regex': 'yahoo'}}, {'source': {'$regex': '.com'}}]}
        ))
        for i in self.the_records:
            matches = [i['source'].find(word)>=0 for word in lst]
            if sum(matches)>0:
                num += 1
        self.assertEqual(qs.count(), num)


    def test_simple_time_reference_query_with_Q_N_OR_iregex_build(self):
        lst = [ 'facebook', 'yahoo', '.com']
        kwargs = {'source__regex': 'blog'}
        q=Q(**kwargs)
        for item in lst:
            kwargs = {'source__iregex': item}
            q |= Q(**kwargs)
        qs = UniqueVisit.objects.filter(q)
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'$or': [{'source': {'$regex': 'blog'}}, {'source': {'$options': 'i', '$regex': 'facebook'}},
                     {'source': {'$options': 'i', '$regex': 'yahoo'}}, {'source': {'$options': 'i', '$regex': '.com'}}]}
        ))
        for i in self.the_records:
            matches = [i['source'].find(word)>=0 for word in lst]
            if sum(matches)>0:
                num += 1
        self.assertEqual(qs.count(), num)


    def test_simple_time_reference_query_with_Q_N_OR_regex_build_negate(self):
        lst = [ 'facebook', 'yahoo', '.com']
        kwargs = {'source__regex': 'blog'}
        q=Q(**kwargs)
        for item in lst:
            kwargs = {'source__regex': item}
            q |= Q(**kwargs)
        qs = UniqueVisit.objects.filter(~q)
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'$nor': [{'source': {'$regex': 'blog'}}, {'source': {'$regex': 'facebook'}},
                      {'source': {'$regex': 'yahoo'}}, {'source': {'$regex': '.com'}}]}
        ))
        for i in self.the_records:
            matches = [i['source'].find(word)>=0 for word in lst]
            if not sum(matches)>0:
                num += 1
        qs = qs.filter(account__in=self.list_of_accounts)
        self.assertEqual(qs.count(), num)

    def test_simple_time_reference_query_with_equal_opr_negate(self):
        qs = UniqueVisit.objects.filter(~Q(first_visit_date =ISODate("2012-07-30T12:29:05Z")))
        num = 0
        self.assertEqual( qs.query.spec, dict({'first_visit_date': {'$ne': datetime(2012, 7, 30, 12, 29, 5, tzinfo=timezone.utc)}}))
        for i in self.the_records:
            if not(i['first_visit_date']  ==  ISODate("2012-07-30T12:29:05Z")):
                num += 1
        qs = qs.filter(account__in=self.list_of_accounts)
        self.assertEqual(qs.count(), num)

    def test_query_with_equal_opr_DOUBLE_negated(self):
        qs = UniqueVisit.objects.filter(~(~Q(first_visit_date =ISODate("2012-07-30T12:29:05Z"))))
        num = 0
        self.assertEqual( qs.query.spec, dict({'first_visit_date': datetime(2012, 7, 30, 12, 29, 5, tzinfo=timezone.utc)}))
        for i in self.the_records:
            if not(not(i['first_visit_date']  ==  ISODate("2012-07-30T12:29:05Z"))):
                num += 1
        self.assertEqual(qs.count(), num)

    def test_simple_time_reference_query_with_Q_lte_opr_negate(self):
        qs = UniqueVisit.objects.filter(~Q(first_visit_date__lte =ISODate("2012-07-30T12:29:05Z")))
        num = 0
        self.assertEqual( qs.query.spec, dict({'first_visit_date': {'$gt': datetime(2012, 7, 30, 12, 29, 5, tzinfo=timezone.utc)}}))
        for i in self.the_records:
            if not(i['first_visit_date'] <= ISODate("2012-07-30T12:29:05Z")):
                num += 1
        qs = qs.filter(account__in=self.list_of_accounts)
        self.assertEqual(qs.count(), num)

    def test_simple_time_reference_query_with_Q_gte_opr_negate(self):
        qs = UniqueVisit.objects.filter(~Q(first_visit_date__gte =ISODate("2012-07-30T12:29:05Z")))
        num = 0
        self.assertEqual( qs.query.spec, dict({'first_visit_date': {'$lt': datetime(2012, 7, 30, 12, 29, 5, tzinfo=timezone.utc)}}))
        for i in self.the_records:
            if not(i['first_visit_date'] >= ISODate("2012-07-30T12:29:05Z")):
                num += 1
        self.assertEqual(qs.count(), num)

    def test_query_with_in_opr_negate(self):
        qs = UniqueVisit.objects.filter(~Q(time_on_site__in=[10, 25, 275]))
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'time_on_site': {'$nin': [10.0, 25.0, 275.0]}}
        ))
        for i in self.the_records:
            if not(i['time_on_site'] == 25 or i['time_on_site'] == 10 or i['time_on_site'] == 275):
                num += 1
        self.assertEqual(qs.count(), num)

    def test_simple_time_reference_query_with_Q_3_AND_negated(self):
        qs = UniqueVisit.objects.filter(~(Q(time_on_site =10)&(~Q(first_visit_date =ISODate("2012-07-30T12:29:05Z")))&
                                          Q(visit_count__lte =2)))
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'$or': [{'time_on_site': {'$ne': 10.0}}, {'visit_count': {'$gt': 2}},
                     {'first_visit_date': datetime(2012, 7, 30, 12, 29, 5, tzinfo=timezone.utc)}]}
        ))
        for i in self.the_records:
            if not(i['time_on_site'] == 10 and
                       not i['first_visit_date'] == ISODate("2012-07-30T12:29:05Z") and
                           i['visit_count']<=2):
                num += 1
        qs = qs.filter(account__in=self.list_of_accounts)
        self.assertEqual(qs.count(), num)


    def test_simple_time_reference_query_with_Q_3_OR_negated(self):
        qs = UniqueVisit.objects.filter(~(Q(time_on_site =10)|Q(first_visit_date =ISODate("2012-07-30T12:29:05Z"))|
                                          Q(visit_count__lte =2)))
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'$nor': [{'time_on_site': 10.0}, {'first_visit_date': datetime(2012, 7, 30, 12, 29, 5, tzinfo=timezone.utc)},
                      {'visit_count': {'$lte': 2}}]}
        ))
        for i in self.the_records:
            if not(i['time_on_site'] == 10 or
                           i['first_visit_date'] == ISODate("2012-07-30T12:29:05Z") or
                           i['visit_count']<=2):
                num += 1
        qs = qs.filter(account__in=self.list_of_accounts)
        self.assertEqual(qs.count(), num)



    def test_dot_notation_query(self):
        qs = UniqueVisit.objects.filter(location__rg__exact ="New York")
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'location.rg': 'New York'}
        ))
        for i in self.the_records:
            if 'rg' in i['location'] and i['location']['rg']  ==  'New York':
                num += 1
        self.assertEqual(qs.count(), num)

    def test_exists_query(self):
        qs = UniqueVisit.objects.filter(demographics__age__exists ="true")
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'demographics.age': {'$exists': 'true'}}
        ))
        for i in self.the_records:
            if 'age' in i['demographics']:
                num += 1
        self.assertEqual(qs.count(), num)

    def test_bson_type_query(self):
        qs = UniqueVisit.objects.filter(landing_page_path__type = int)
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'landing_page_path': {'$type': 16}}
        ))
        for i in self.the_records:
            if type( i['landing_page_path']) == int:
                num += 1
        self.assertEqual(qs.count(), num)

        qs = UniqueVisit.objects.filter(landing_page_path__type = str)
        num = 0
        self.assertEqual( qs.query.spec, dict(
            {'landing_page_path': {'$type': 2}}
        ))
        for i in self.the_records:
            if type( i['landing_page_path']) == str:
                num += 1
        self.assertEqual(qs.count(), num)


class TestMongoORMInfrastructure(TestCase):
    def setUp(self):
        self.user = Profile()
        self.user.username = 'testuser'
        self.user.company_name = 'Test Company'
        self.user.set_password('testuser')
        self.user.save()

        self.acc = AnalyticsAccount(user = self.user)
        self.acc.pk = 10000
        self.acc.name = 'Test'
        self.acc.analytics_id = 'asd324sf456m6n76b5b'
        self.acc.history_fetched = False
        self.acc.save()

        self.other_acc = AnalyticsAccount(user = self.user)
        self.other_acc.pk = 10001
        self.other_acc.name = 'Test1'
        self.other_acc.analytics_id = 'asd324sf456m111111'
        self.other_acc.history_fetched = False
        self.other_acc.save()

        self.list_of_accounts=[self.acc, self.other_acc]

        records = []

        records.append({"start_date":datetime(2005, 7, 28), "values":[], "some_id":2,})
        records.append({"start_date":datetime(2005, 8, 28), "values":[], "some_id":2,})
        records.append({"start_date":datetime(2005, 8, 28), "values":[], "some_id":3,})

        self.the_records = records

        TestModel1.objects.all().delete()
        self.assertEqual(TestModel1.objects.all().count(), 0)
        TestModel2.objects.all().delete()
        self.assertEqual(TestModel2.objects.all().count(), 0)
        TestModel3.objects.all().delete()
        self.assertEqual(TestModel3.objects.all().count(), 0)

        TestModel1.objects.bulk_insert(self.the_records)

        self.assertEqual(TestModel1.objects.all().count(), len(self.the_records))

        tm1a = TestModel1.objects.get(start_date = datetime(2005, 7, 28, tzinfo=timezone.utc), some_id =2)

        # Creating on object from TestModel2 referencing an object from TestModel1
        tm2a = TestModel2(ref1= tm1a, start_date = datetime(2005, 8, 12), values=[1, 3], some_id =12)
        tm2a.save()

        self.assertEqual(TestModel2.objects.all().count(), 1)

        # Creating on object from TestModel3 wo reference
        tm3a = TestModel3(start_date = datetime(2005, 8, 12), values=[1, 3], some_id =12)
        tm3a.save()
        self.assertEqual(TestModel3.objects.all().count(), 1)

        # Creating on object from TestModel3 with reference
        tm1b = TestModel1.objects.get(start_date = datetime(2005, 8, 28, tzinfo=timezone.utc), some_id =3)
        tm2b = TestModel2(ref1= tm1b, start_date = datetime(2005, 8, 13), values=[1, 3], some_id =13)
        tm2b.save()
        tm3b = TestModel3( ref2= tm2b, start_date = datetime(2005, 8, 12), values=[1, 3], some_id =14)
        tm3b.save()
        self.assertEqual(TestModel3.objects.all().count(), 2)
        self.assertEqual(TestModel2.objects.all().count(), 2)


    def test_delete_non_referenced_object(self):
        pre1 = TestModel1.objects.all().count()
        self.assertNotEqual(pre1, 0)
        pre2 = TestModel2.objects.all().count()
        self.assertNotEqual(pre2, 0)
        pre3 = TestModel3.objects.all().count()
        self.assertNotEqual(pre3, 0)
        TestModel1.objects.filter(start_date = datetime(2005, 8, 28, tzinfo=timezone.utc), some_id =3).delete()

        self.assertEqual(TestModel1.objects.all().count(), pre1-1)
        self.assertEqual(pre2, TestModel2.objects.all().count())
        self.assertEqual(pre3, TestModel3.objects.all().count())


    def test_delete_referenced_object_m1_m2_with_cascade(self):
        pre1 = TestModel1.objects.all().count()
        self.assertNotEqual(pre1, 0)
        pre2 = TestModel2.objects.all().count()
        self.assertNotEqual(pre2, 0)
        pre3 = TestModel3.objects.all().count()
        self.assertNotEqual(pre3, 0)
        tm1o =  TestModel1.objects.get(start_date = datetime(2005, 7, 28, tzinfo=timezone.utc), some_id =2)
        tm1o.delete()

        self.assertEqual(TestModel1.objects.all().count(), pre1-1)
        self.assertEqual(TestModel2.objects.all().count(), pre2-1)
        self.assertEqual(pre3, TestModel3.objects.all().count())

    def test_delete_of_queryset(self):
        pre1 = TestModel1.objects.all().count()
        self.assertNotEqual(pre1, 0)
        pre2 = TestModel2.objects.all().count()
        self.assertNotEqual(pre2, 0)
        pre3 = TestModel3.objects.all().count()
        self.assertNotEqual(pre3, 0)
        TestModel1.objects.filter(start_date = datetime(2005, 7, 28, tzinfo=timezone.utc), some_id =2).delete()

        self.assertEqual(TestModel1.objects.all().count(), pre1-1)
        # Delete through the filter will behave differently and will not follow a cascade relationship
        # (see different behaviour in test_delete_referenced_object_m1_m2)
        self.assertEqual(TestModel2.objects.all().count(), pre2)
        self.assertEqual(pre3, TestModel3.objects.all().count())


    def test_delete_cascade_2nd_level(self):
        pre1 = TestModel1.objects.all().count()
        self.assertNotEqual(pre1, 0)
        pre2 = TestModel2.objects.all().count()
        self.assertNotEqual(pre2, 0)
        pre3 = TestModel3.objects.all().count()
        self.assertNotEqual(pre3, 0)
        tm1o =  TestModel1.objects.get(start_date = datetime(2005, 8, 28, tzinfo=timezone.utc), some_id =3)
        tm1o.delete()

        self.assertEqual(TestModel1.objects.all().count(), pre1-1)
        self.assertEqual(TestModel2.objects.all().count(), pre2-1)
        self.assertEqual(TestModel3.objects.all().count(), pre3-1)


def ISODate(timestr):
    res = datetime.strptime(timestr, "%Y-%m-%dT%H:%M:%SZ")
    res = res.replace(tzinfo=timezone.utc)
    return res

def ObjectId(st):
    return st




from django.template import Context
from django.template.loader import get_template
from django import template

register = template.Library()


@register.filter
def bootstrap(element):
    element_type = element.__class__.__name__.lower()
    if element_type == 'boundfield':
        template = get_template("bootstrapform/field.html")
        context = Context({'field': element})
    else:
        template = get_template("bootstrapform/form.html")
        context = Context({'form': element})

    return template.render(context)


@register.filter
def is_checkbox(field):
    return field.field.widget.__class__.__name__.lower() == "checkboxinput"


@register.filter
def is_radio(field):
    return field.field.widget.__class__.__name__.lower() == "radioselect"



#==========================================================================
# Copyright 2012 Lucidel, Inc., 2013 Cloudoscope Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================

"""
WSGI config for moma_example project.

This module contains the WSGI application used by Django's development server
and any production WSGI deployments. It should expose a module-level variable
named ``application``. Django's ``runserver`` and ``runfcgi`` commands discover
this application via the ``WSGI_APPLICATION`` setting.

Usually you will have the standard Django WSGI application here, but it also
might make sense to replace the whole Django WSGI application with a custom one
that later delegates to the Django one. For example, you could introduce WSGI
middleware here, or combine a Django application with an application of another
framework.

"""

import os

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "moma_example.settings")

# This application object is used by any WSGI server configured to use this
# file. This includes Django's development server, if the WSGI_APPLICATION
# setting points here.
from django.core.wsgi import get_wsgi_application
application = get_wsgi_application()

# Apply WSGI middleware here.
# from helloworld.wsgi import HelloWorldApplication
# application = HelloWorldApplication(application)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys


try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages


version = '0.1.1'

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    sys.exit()

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

setup(
    name='moma-django',
    version='0.1.1',
    description='MoMa-Django provides native Django ORM and admin support for Mongo DB.',
    long_description=readme + '\n\n' + history,
    author='Gadi Oren',
    author_email='gadi.oren.1@gmail.com',
    url='https://github.com/gadio/moma-django',
    packages=[
        'moma_django', 'moma_django.management.commands',
        'moma_example', 'moma_example.moma_example', 'moma_example.testing', 'moma_example.data',
        'moma_example.bootstrapform', 'moma_example.bootstrapform.templatetags',
    ],
    package_dir={'moma_django': 'moma_django', 'moma_example': 'moma_example'},
    include_package_data=True,
    install_requires=[
       'pymongo>=2.1.1', 'django>=1.4, <1.5', 'djangotoolbox>=1.4.0',
    ],
    package_data={'moma_example':
                        [
                          'moma_example/media/bootstrap/css/*.css', 'media/bootstrap/css/*.map',
                          'media/bootstrap/fonts/*.*', 'media/bootstrap/js/*.js',
                          'templates/*.html', 'moma_example/templates/question/*.html',
                        ],
                  },
    license="GPL",
    zip_safe=False,
    keywords='moma-django,mongodb,django',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    test_suite='tests',
)



#==========================================================================
# Copyright 2012 Lucidel, Inc., 2013 Cloudoscope Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================
from django import forms
from django.forms import HiddenInput


class QuetionEditForm(forms.Form):
    question_txt = forms.CharField(required=True, label="question_txt", widget=forms.Textarea())
    question_id = forms.CharField(required=False, label="question id",
                                 widget = forms.TextInput(attrs={'type': 'hidden',}))

    def __init__(self, *args, **kwargs):
        from django.forms.widgets import HiddenInput
        hide_condition = kwargs.pop('hide_condition',None)
        super(forms.Form, self).__init__(*args, **kwargs)
        if hide_condition:
            self.fields['question_id'].widget = HiddenInput()


class DocumentForm(forms.Form):
    docfile = forms.FileField(required=True,
        label='Upload an Image',
        help_text='Note: on a device with a camera, pressing "Choose File" would allow taking a picture. Other devices would enable image selection (not too large please...)',
        widget=forms.FileInput(attrs={'capture':"camera", 'accept':"image/*"}),
    )
    question_id = forms.CharField(required=True, label="question id",
                                  widget = forms.TextInput(
                                      attrs={'type': 'hidden',}
                                  ))

    def __init__(self, *args, **kwargs):
        from django.forms.widgets import HiddenInput
        hide_condition = kwargs.pop('hide_condition',None)
        super(forms.Form, self).__init__(*args, **kwargs)
        if hide_condition:
            self.fields['question_id'].widget = HiddenInput()




#==========================================================================
# Copyright 2012 Lucidel, Inc., 2013 Cloudoscope Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this work except in compliance with the License.
# You may obtain a copy of the License in the LICENSE file, or at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================
from django.contrib.auth.models import User

from django.db import models
from moma_django.models import MongoModel, post_syncdb_mongo_handler
from moma_django.fields import MongoDateTimeField, DictionaryField, ValuesField



# Enabling South for the non conventional mongo model
#
# add_introspection_rules(
#     [
#         (
#             (MongoIdField, MongoDateTimeField, DictionaryField ),
#             [],
#             {
#                 "max_length":   ["max_length",   {"default": None}],
#                 },
#         ),
#         ],
#     ["^mongo_django.fields.*",])


class Question(MongoModel):
    user = models.ForeignKey(User)
    date = MongoDateTimeField(db_index=True)
    question = models.CharField(max_length=256 )

    docs = DictionaryField(models.CharField())
    image = DictionaryField(models.TextField())
    audio = DictionaryField()
    other = DictionaryField()

    vote_ids = ValuesField(models.IntegerField())


    def __unicode__(self):
        return u'%s[%s %s]' % (self.question, self.date, self.user, )

    class Meta:
        unique_together = ['user', 'question',]
        managed = False

models.signals.post_syncdb.connect(post_syncdb_mongo_handler)

